import * as Rx from 'rxjs/Observable'
import * as curio from '@curio/client'
import { HttpConnection } from 'curiojs'
import config from '../config'

curio.setObservableConstructor(Rx.Observable)

const connection = new HttpConnection(config().service)
connection.debug = false

/**
 * the sink of truth
 */
export const sink = new curio.Dispatcher<curio.Message>()

/**
 * the source of truth
 */
export const source = sink
.pipe(
  curio.operators.map((message) => {
    switch (message.signalName) {
      case curio.constants.OPEN_SESSION_SIGNAL_NAME: return message.recycle({
        body: {
          SYSTEM_CODE: config().service.system,
          LOGIN: message.body.login,
          PASSWORD: message.body.password
        }
      })
      default: return message
    }
  }),
  connect(connection)
)

/**
 * the service itself
 */
export const service = new curio.Service(source.pipe(
  curio.operators.map((message) => {
    if (message.error) return message
    switch (message.signalName) {
      case curio.constants.OPEN_SESSION_SIGNAL_NAME: return message.recycle({
        body: Number(message.body.UCID[0]._)
      })
      default: return message
    }
  })
))

service.subscribe(sink)

function connect (conn: HttpConnection): curio.Operator<curio.Message> {
  return (source) => new curio.Observable((subscriber) => source.subscribe((request) => {
    conn.request(request.recipient, request.signalName, request.body)
    .then((response) => request.recycle({
      recipient: request.sender,
      sender: request.recipient,
      body: response
    }))
    .catch((error) => request.recycle({
      recipient: request.sender,
      sender: request.recipient,
      body: null,
      error
    }))
    .then(subscriber.next.bind(subscriber))
  }))
}
